// Squawk messenger.
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CORE_DATABASE_EXCEPTIONS_H
#define CORE_DATABASE_EXCEPTIONS_H

#include "exception.h"
#include "database.h"

namespace Core {

class DataBase::Directory: public Utils::Exception {
public:
    Directory(const std::string& p_path):Exception(), path(p_path){}

    std::string getMessage() const{return "Can't create directory for database at " + path;}
private:
    std::string path;
};

class DataBase::Closed : public Utils::Exception {
public:
    Closed(const std::string& op, const std::string& acc):Exception(), operation(op), account(acc){}

    std::string getMessage() const{return "An attempt to perform operation " + operation + " on closed archive for " + account;}
private:
    std::string operation;
    std::string account;
};

class DataBase::Opened : public Utils::Exception {
public:
    Opened(const std::string& tn):Exception(), tableName(tn){}

    std::string getMessage() const{return "An attempt to add table " + tableName + " but it's impossible to do if the DataBase is already opened";}
private:
    std::string tableName;
};

class DataBase::NotFound : public Utils::Exception {
public:
    NotFound(const std::string& k, const std::string& acc):Exception(), key(k), account(acc){}

    std::string getMessage() const{return "Element for id " + key + " wasn't found in database " + account;}
private:
    std::string key;
    std::string account;
};

class DataBase::Empty : public Utils::Exception {
public:
    Empty(const std::string& acc):Exception(), account(acc){}

    std::string getMessage() const{return "An attempt to read ordered elements from database " + account + " but it's empty";}
private:
    std::string account;
};

class DataBase::Exist : public Utils::Exception {
public:
    Exist(const std::string& acc, const std::string& p_key):Exception(), account(acc), key(p_key){}

    std::string getMessage() const{return "An attempt to insert element " + key + " to database " + account + " but it already has an element with given id";}
private:
    std::string account;
    std::string key;
};

class DataBase::NoAvatar : public Utils::Exception {
public:
    NoAvatar(const std::string& el, const std::string& res):Exception(), element(el), resource(res){
        if (resource.size() == 0) {
            resource = "for himself";
        }
    }

    std::string getMessage() const{return "Element " + element + " has no avatar for " + resource ;}
private:
    std::string element;
    std::string resource;
};

class DataBase::Unknown : public Utils::Exception {
public:
    Unknown(const std::string& acc, const std::string& message):Exception(), account(acc), msg(message){}

    std::string getMessage() const{return "Unknown error on database " + account + ": " + msg;}
private:
    std::string account;
    std::string msg;
};
}
#endif //CORE_DATABASE_EXCEPTIONS_H
